'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var resolve = require('../resolve');
var symbols = require('../symbols');
var getFilledArray = require('../getFilledArray');
var stringLength = require('string-length');

var calculateMaxCellsWidth = function calculateMaxCellsWidth(colsMaxWidth, maxWidth) {
	var maxWidthCols = 0;
	colsMaxWidth.forEach(function (width) {
		return maxWidthCols += width;
	});

	// if enough space is available
	if (maxWidthCols <= maxWidth) {
		// fill array with still available space
		var currentIndex = 0;
		getFilledArray(maxWidth - maxWidthCols, '').forEach(function () {
			colsMaxWidth[currentIndex] += 1;

			currentIndex += 1;
			if (currentIndex >= colsMaxWidth.length) {
				currentIndex = 0;
			}
		});
		return colsMaxWidth;
	}

	var out = getFilledArray(colsMaxWidth.length, 0);
	var counterAlreadyUsed = 0;

	// share available space on cols
	getFilledArray(maxWidth, '').forEach(function () {
		out.forEach(function (item, index) {
			if (out[index] < colsMaxWidth[index] && counterAlreadyUsed < maxWidth) {
				counterAlreadyUsed += 1;
				out[index] += 1;
			}
		});
	});
	return out;
};

var getTablesArray = function getTablesArray(markdown) {
	// eslint-disable-next-line
	var reg = /\|.*/g;
	var results = void 0;
	var resultsArr = [];
	var tables = [];
	var currentTableIndex = -1;

	// eslint-disable-next-line
	while ((results = reg.exec(markdown)) !== null) {
		resultsArr.push({
			line: results[0],
			isSeparator: Boolean(results[0].indexOf('---') > 0),
			index: results.index
		});
	}

	/* eslint complexity: 0*/
	resultsArr.forEach(function (_ref, resultsArrIndex) {
		var line = _ref.line,
		    index = _ref.index,
		    isSeparator = _ref.isSeparator;

		var isFirstLine = false;

		// create new table if second line is separator
		if (resultsArr[resultsArrIndex + 1] && resultsArr[resultsArrIndex + 1].isSeparator) {
			currentTableIndex += 1;
			isFirstLine = true;
			tables.push({ maxCellsWidht: [], headerRows: [], bodyRows: [], startIndex: index });
		}

		var table = tables[currentTableIndex];

		// detect if last line
		if (resultsArrIndex + 1 === resultsArr.length || resultsArr[resultsArrIndex + 2] && resultsArr[resultsArrIndex + 2].isSeparator) {
			table.endIndex = index + resultsArr[resultsArrIndex].line.length;
		}

		if (isSeparator) {
			return;
		}

		var rows = isFirstLine ? table.headerRows : table.bodyRows;
		rows.push({
			line: line,
			index: index,
			cells: []
		});

		var cells = line.split('|');

		// pase cells
		cells.forEach(function (cellValue, cellIndex) {
			if (cellIndex === 0 || cellIndex === cells.length - 1) {
				return;
			}

			// create new maxLength array item
			if (table.maxCellsWidht[cellIndex - 1] === undefined) {
				table.maxCellsWidht.push(0);
			}
			cellValue = cellValue.trim();
			rows[rows.length - 1].cells.push(cellValue);

			// push max length
			if (cellValue.length > table.maxCellsWidht[cellIndex - 1]) {
				table.maxCellsWidht[cellIndex - 1] = cellValue.length;
			}
		});
	});

	return tables;
};

var roundUp = function roundUp(value) {
	if (value.toString().indexOf('.') < 0) {
		return value;
	}
	return Math.floor(value) + 1;
};

var Row = function () {
	function Row(cells, calcMaxCellsWidth, symbolsObj) {
		_classCallCheck(this, Row);

		this.cells = cells;
		this.calcMaxCellsWidth = calcMaxCellsWidth;
		this.symbols = symbolsObj;
	}

	_createClass(Row, [{
		key: 'lines',
		get: function get() {
			var _this = this;

			var lines = getFilledArray(this.linesCount, '');
			var availableSpaces = [].concat(this.calcMaxCellsWidth);
			var alradyFilled = getFilledArray(this.calcMaxCellsWidth.length, 0);

			lines.forEach(function (_, lineIndex) {
				var line = [];
				_this.cells.forEach(function (cell, cellIndex) {
					var availableSpace = availableSpaces[cellIndex];
					var sliced = cell.slice(alradyFilled[cellIndex], availableSpace * (lineIndex + 1));
					line.push(_this.symbols.cell(sliced + getFilledArray(availableSpace - stringLength(sliced), ' ').join('')));
					alradyFilled[cellIndex] = alradyFilled[cellIndex] + availableSpace;
				});
				lines[lineIndex] = '' + _this.symbols.left + line.join(_this.symbols.vSeparator) + _this.symbols.right;
			});

			return lines.join('\n');
		}
	}, {
		key: 'linesCount',
		get: function get() {
			var availableSpaces = this.calcMaxCellsWidth;
			var max = 0;
			this.cells.forEach(function (cell, index) {
				var count = roundUp(cell.length / availableSpaces[index]);
				if (count > max) {
					max = count;
				}
			});
			return max;
		}
	}]);

	return Row;
}();

var RowSeparator = function () {
	function RowSeparator(calcMaxCellsWidth, width, symbolsObj) {
		_classCallCheck(this, RowSeparator);

		this.calcMaxCellsWidth = calcMaxCellsWidth;
		this.symbols = symbolsObj;
		this.width = width;
	}

	_createClass(RowSeparator, [{
		key: 'innerWidth',
		get: function get() {
			return this.width - stringLength(this.symbols.left) - stringLength(this.symbols.right);
		}
	}, {
		key: 'separator',
		get: function get() {
			return '' + this.symbols.left + getFilledArray(this.innerWidth, this.symbols.cell).join('') + this.symbols.right;
		}
	}]);

	return RowSeparator;
}();

/* eslint require-jsdoc: 0*/


var ConvertMarkdownTable = function () {
	function ConvertMarkdownTable(table, width) {
		var _this2 = this;

		_classCallCheck(this, ConvertMarkdownTable);

		this.table = table;
		this.width = width;
		this.maxCellsWidht = table.maxCellsWidht;
		this.calcMaxCellsWidth = calculateMaxCellsWidth(this.maxCellsWidht, this.width - this.rowSymbolsWidth);
		this.lines = [];

		var rowTop = new RowSeparator(this.calcMaxCellsWidth, this.width, this.symbolsTop);
		this.lines.push(rowTop.separator);

		this.table.headerRows.forEach(function (_ref2) {
			var cells = _ref2.cells;

			var row = new Row(cells, _this2.calcMaxCellsWidth, _this2.symbolsHeader);
			_this2.lines.push(row.lines);
		});
		this.table.bodyRows.forEach(function (_ref3) {
			var cells = _ref3.cells;

			var rowSerarator = new RowSeparator(_this2.calcMaxCellsWidth, _this2.width, _this2.symbolsSeparator);
			_this2.lines.push(rowSerarator.separator);

			var row = new Row(cells, _this2.calcMaxCellsWidth, _this2.symbolsBody);
			_this2.lines.push(row.lines);
		});

		var rowBottom = new RowSeparator(this.calcMaxCellsWidth, this.width, this.symbolsBottom);
		this.lines.push(rowBottom.separator);
	}

	_createClass(ConvertMarkdownTable, [{
		key: 'cliTable',
		get: function get() {
			return '' + this.lines.join('\n');
		}
	}, {
		key: 'rowSymbolsWidth',
		get: function get() {
			var _symbolsHeader = this.symbolsHeader,
			    left = _symbolsHeader.left,
			    cell = _symbolsHeader.cell,
			    right = _symbolsHeader.right,
			    vSeparator = _symbolsHeader.vSeparator;

			return stringLength(left) + stringLength(right) + stringLength(cell('')) * this.maxCellsWidht.length + stringLength(vSeparator) * (this.maxCellsWidht.length - 1);
		}
	}, {
		key: 'symbolsHeader',
		get: function get() {
			return {
				left: symbols.table.headerStart,
				cell: symbols.table.headerCell,
				right: symbols.table.headerEnd,
				vSeparator: symbols.table.headerVSeparator
			};
		}
	}, {
		key: 'symbolsBody',
		get: function get() {
			return {
				left: symbols.table.bodyStart,
				cell: symbols.table.bodyCell,
				right: symbols.table.bodyEnd,
				vSeparator: symbols.table.bodyVSeparator
			};
		}
	}, {
		key: 'symbolsTop',
		get: function get() {
			return {
				left: symbols.table.topStart,
				cell: symbols.table.topCell,
				right: symbols.table.topEnd
			};
		}
	}, {
		key: 'symbolsBottom',
		get: function get() {
			return {
				left: symbols.table.bottomStart,
				cell: symbols.table.bottomCell,
				right: symbols.table.bottomEnd
			};
		}
	}, {
		key: 'symbolsSeparator',
		get: function get() {
			return {
				left: symbols.table.hSeparatorStart,
				cell: symbols.table.hSeparatorCell,
				right: symbols.table.hSeparatorEnd
			};
		}
	}]);

	return ConvertMarkdownTable;
}();

/**
 * @param {string} markdown - the readme files data
 * @param {Object} di - dependency injection
 * @returns {string} markdown - replaced readme file data
 **/


function convertMarkdownTableToCli(markdown, _ref4) {
	var _process = _ref4._process;

	var tables = getTablesArray(markdown);
	var originMarkdown = '' + markdown;
	var width = _process.stdout.columns;

	tables.forEach(function (table) {
		var convertedTable = new ConvertMarkdownTable(table, width);
		markdown = markdown.replace(originMarkdown.substring(table.startIndex, table.endIndex), convertedTable.cliTable);
	});
	return markdown;
}

module.exports = resolve(convertMarkdownTableToCli, { process: process });
module.exports.convertMarkdownTableToCli = convertMarkdownTableToCli;
module.exports.calculateMaxCellsWidth = calculateMaxCellsWidth;